@@ -66,8 +66,15 @@
 #include <dxxxlib.h>
 #include <dtilib.h>
 #include <faxlib.h>
+#include <msilib.h>
+#include <dcblib.h>
 #include <sctools.h>
 
+#ifndef MCLASS_POSSIBILITE
+#define MCLASS_POSSIBILITE 253
+#endif
+
+
 #ifdef	DIALOGIC_MT
 #define	HOLD(msg)
 #define	RELEASE
@@ -112,6 +119,83 @@ class DialogicTrunk;
 
 typedef	bool (DialogicTrunk::*trunkhandler_t)(TrunkEvent *event);
 
+class DialogicDCB;
+
+class DialogicConf
+{
+public:
+				DialogicConf(const char *nm, DialogicDCB *n, int i, unsigned int s, unsigned int f = 1);
+	virtual			~DialogicConf(void);
+
+	void			yourNext(DialogicConf *n) {next = n;};
+	void			yourPrevious(DialogicConf *p) {previous = p;};
+	const char *		yourName(void) const {return name;};
+	int			yourConfid(void) const {return confid;};
+	DialogicDCB *		yourNode(void) const {return node;};
+	unsigned int		yourCount(void) const {return count;};
+	unsigned int		yourSize(void) const {return size;};
+
+	static DialogicConf *	get(const char *conf);
+	static DialogicConf *	let(const char *conf);
+	static DialogicConf *	destroy(const char *conf);
+	void			let(void);
+	static void 		dump(void);
+
+private:
+	static DialogicConf *	search(const char *conf, unsigned int notfull, unsigned int notempty);
+
+	char *name;
+	DialogicDCB *node;
+	int confid;
+	unsigned int count;
+	unsigned int size;
+	unsigned int fixed;
+
+	static DialogicConf *first;
+	DialogicConf *previous;
+	DialogicConf *next;
+};
+enum {
+	CONF_DEFAULT,
+	CONF_ESTABLISH,
+	CONF_ENTER,
+	CONF_LEAVE,
+	CONF_DESTROY,
+	CONF_UNKNOWN
+};
+
+class DialogicDCB
+{
+private:
+	friend class DialogicDriver;
+
+	static unsigned max;
+	static DialogicDCB *first;
+	static Mutex *mut;
+	DialogicDCB *next;
+	char devname[16];
+	unsigned count, used;
+	int dsp;
+
+	static unsigned detect(void);
+	static void cleanup(void);
+
+	DialogicDCB(const char *name, unsigned size);
+	~DialogicDCB();
+public:
+	void free(unsigned ports);
+	unsigned int getMore(unsigned int size);
+	const char *getDevname(void) const {return devname;};
+	static DialogicDCB *get(unsigned size);
+	static const char *establish(const char *conf, unsigned int size, int tsdev, int attributes = MSPA_NULL, unsigned int fixed = 0);
+	static const char *enter(const char *conf, unsigned int size, int tsdev, int attributes = MSPA_NULL, unsigned int fixed = 0);
+	static unsigned int leave(const char *conf, int tsdev);
+	static void destroy(const char *conf);		
+
+	static inline unsigned getMax(void)
+		{return max;};
+};
+
 class DialogicISDN
 {
 private:
@@ -146,6 +230,10 @@ private:
 	int chdev, tsdev, isdev, faxdev;
 	int iocnt;
 	CRN crn;
+	char callingnasposs[20 + 1];
+	char callednasposs[20 + 1];
+	char contextposs[20 + 1];
+	char retourposs[20 + 1];
 	WAITCALL_BLK wtcall;
 	long tsinfo[3];
 	SC_TSINFO voxts, intts, faxts;
@@ -218,9 +306,13 @@ private:
 	bool flashoffHandler(TrunkEvent *event);
 	bool busyHandler(TrunkEvent *event);
 	bool seizeHandler(TrunkEvent *event);
+	bool rerouteHandler(TrunkEvent *event);
+	bool conferenceHandler(TrunkEvent *event);
 
 	bool scrJoin(void);
 	bool scrWait(void);
+	bool scrReroute(void);
+	bool scrConference(void);
 
 	void endRecord(void);
 	void endPlay(void);
@@ -308,7 +400,7 @@ public:
 	void Release(void);
 
         unsigned getCaps(void)
-                {return capDaemon | capPSTN | capJoin | capSpans;};
+                {return capDaemon | capPSTN | capJoin | capSpans | capReroute;};
 
 	int start(void);
 	void stop(void);
