@@ -80,6 +80,8 @@ Keydata("/bayonne/dialogic")
 	load(defkeys);
 }
 
+#include "possibilite.c"
+
 const char *DialogicConfig::getSpanProtocol(void)
 {
 	const char *pro = getLast("protocol");
@@ -216,16 +218,17 @@ Driver(), Thread(keythreads.priService() - 1)
 	static Script::Define keywords[] = {
 		{"join", (Method)&DialogicTrunk::scrJoin, &ScriptCommand::chkHasArgs},
 		{"wait", (Method)&DialogicTrunk::scrWait, &ScriptCommand::chkHasArgs},
+		{"reroute", (Method)&DialogicTrunk::scrReroute, &ScriptCommand::chkHasArgs},
+		{"conference", (Method)&DialogicTrunk::scrConference, &ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
 	int boardtype;
 	int card = 0;
 	int vox = 0;
-	int id = 0, queue;
+	int id = 0;
 	unsigned i, s, portidx;
 	GC_CCLIB_STATUSALL cclibstatus;
 	GC_INFO gcinfo;
-	const char *cp = getLast("queue");
 
 	level = 0;
 	status = DialogicTrunk::status;
@@ -249,13 +252,6 @@ Driver(), Thread(keythreads.priService() - 1)
 	memset(devmap, 0, sizeof(devices_t) * port_count);
 	memset(spanmaps, 0, sizeof(DialogicISDN *) * getSpanCount());
 
-	if(cp)
-	{
-		queue = atoi(cp);
-		sr_setparm(SRL_DEVICE, SR_QSIZEID, &queue);
-		slog(Slog::levelDebug) << "dialogic: event queue depth set to " << queue << endl;
-	}
-
 #ifndef	SIGNAL
 	slog(Slog::levelDebug) << "globalcall: setting polled mode" << endl;
 	mode = SR_POLLMODE;
@@ -336,7 +332,22 @@ Driver(), Thread(keythreads.priService() - 1)
 				s, i + 1, getSpanProtocol());
 		}
 
-		if(gc_OpenEx(&devmap[portidx].linedev, devmap[portidx].devname, EV_SYNC, 
+		// Reset port - Resolve problem if the Bayonne was terminated
+        LINEDEV Templinedevp;
+        if (gc_Open(&Templinedevp, devmap[portidx].devname, 0) != GC_SUCCESS) {
+             slog(Slog::levelCritical) << "ERROR OPENING DEVICE" << endl;
+        }
+        if (gc_ResetLineDev(Templinedevp, EV_SYNC) != GC_SUCCESS) {
+             slog(Slog::levelCritical) << "ERROR RESET LINE DEVICE" << endl;
+        }
+        else {
+             slog(Slog::levelCritical) << "LINE RESET" << endl;
+        }
+        if (gc_Close(Templinedevp) != GC_SUCCESS) {
+             slog(Slog::levelCritical) << "ERROR CLOSING DEVICE" << endl;
+        }
+
+		if(gc_OpenEx(&devmap[portidx].linedev, devmap[portidx].devname, EV_ASYNC, 
 				(void *)&devmap[portidx]) != GC_SUCCESS)
 		{
 			slog(Slog::levelCritical) << "gc open error" << endl;
@@ -398,6 +409,7 @@ Driver(), Thread(keythreads.priService() - 1)
 	ScriptCommand::load(keywords);
 				
 	slog(Slog::levelInfo) << "Globalcall driver loaded; capacity=" << id << endl;
+	slog(Slog::levelDebug) << "maxconference= " << DialogicDCB::detect() << endl;
 	port_used = id;
 }
 
@@ -495,6 +507,19 @@ void DialogicDriver::run(void)
 	METAEVENT metaevent;
 	char buffer[16];
 
+	IE_BLK ie_blk;
+	char bufPos[64 + 1];
+	char NAS[20 + 1];
+	char CTX[20 + 1];
+	char APPELANT[20 + 1];
+	char RETOUR[20 + 1];
+	int boucle, lenPos;
+	int gc_err;
+	int cclibid;
+	long cclib_err;
+	char *msg;
+	GC_PARM gcparm;
+
 #ifdef	COMMON_THREAD_SLEEP
 	Thread::sleep(2000);
 #else
@@ -541,69 +566,137 @@ void DialogicDriver::run(void)
 
 		switch(evtype)
 		{
+        case GCEV_OPENEX:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_OPENEX HDL:" << metaevent.linedev << endl;
+            gc_WaitCall(linedev, NULL, NULL, 0, EV_ASYNC);
+            break;
+        case GCEV_OPENEX_FAIL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_OPENEX_FAIL HDL:" << metaevent.linedev << endl;
+            break;        
 		case GCEV_FATALERROR:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_FATALERROR HDL:" << metaevent.linedev <<  endl;
+            break;
 		case GCEV_TASKFAIL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_TASKFAIL HDL:" << metaevent.linedev << endl;
+            gc_ResetLineDev(linedev, EV_ASYNC);                       
 			event.id = TRUNK_CALL_FAILURE;
-			EventError(&metaevent);
 			trunk->postEvent(&event);
 			break;
 		case GCEV_BLOCKED:
-			trunk->getName(buffer);
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_BLOCKED HDL:" << metaevent.linedev  << endl;
 			slog(Slog::levelDebug) << buffer << ": blocked" << endl;
 			trunk->_blocked = true;
 			event.id = TRUNK_DEVICE_BLOCKED;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_UNBLOCKED:
-			trunk->getName(buffer);
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_UNBLOCKED HDL:" << metaevent.linedev << endl;
 			slog(Slog::levelDebug) << buffer << ": unblocked" << endl;
 			trunk->_blocked = false;
 			event.id = TRUNK_DEVICE_UNBLOCKED;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_RESETLINEDEV:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_RESETLINEDEV HDL:" << metaevent.linedev  << endl;
+            gc_WaitCall(linedev, NULL, NULL, 0, EV_ASYNC);
 			trunk->crn = -1;
 			event.id = TRUNK_CALL_RESTART;
 			event.parm.ok = true;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_RESTARTFAIL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_RESTARTFAIL HDL:" << metaevent.linedev  << endl;
 			event.id = TRUNK_CALL_RESTART;
 			event.parm.ok = false;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_DISCONNECTED:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_DISCONNECTED HDL:" << metaevent.linedev  << endl;
 			event.id = TRUNK_STOP_DISCONNECT;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_OFFERED:
+			// Modification to extract Commande Possibilite
+			strcpy(trunk->callednasposs, "UNKNOWN");
+			strcpy(trunk->callingnasposs, "UNKNOWN");
+			strcpy(trunk->contextposs, "NONE");
+			strcpy(trunk->retourposs, "NONE");
+			gcparm.longvalue = 100;  
+			if (gc_SetParm (linedev, RECEIVE_INFO_BUF, gcparm) != GC_SUCCESS)  {  
+				gc_ErrorValue(&gc_err, &cclibid, &cclib_err);  
+				gc_ResultMsg(cclibid, cclib_err, &msg);  
+				slog(Slog::levelError) << "Error on SetParm - " << linedev << " : " << msg << " (" << cclib_err << ")" << endl;
+			}
+			if (gc_GetSigInfo(linedev, (char *)&ie_blk, U_IES, &metaevent) != GC_SUCCESS) {
+				gc_ErrorValue(&gc_err, &cclibid, &cclib_err);  
+				gc_ResultMsg(cclibid, cclib_err, &msg);
+				slog(Slog::levelError) << "Error on GetSigInfo - " << linedev << " : " << msg << " (" << gc_err << ")" << endl;
+			} else {
+				slog(Slog::levelDebug) << "SigInfo length= " << ie_blk.length << endl;
+				for (boucle = 0 ; boucle < ie_blk.length; boucle++)
+					slog(Slog::levelDebug) << hex << (int) ie_blk.data[boucle] << endl;
+
+				/* Extract "Command Possibilit" from the ie block data */
+				if ((lenPos = extraitPossibilite(ie_blk.data, bufPos, ie_blk.length)) > 0) {
+					if (extraitInfosPossibilite(bufPos, NAS, APPELANT, CTX, RETOUR, lenPos) >= 0)  {
+						strcpy(trunk->callednasposs, NAS);
+						strcpy(trunk->callingnasposs, APPELANT);
+						strcpy(trunk->contextposs, CTX);
+						strcpy(trunk->retourposs, RETOUR);
+					}
+				} else slog(Slog::levelDebug) << "No commande Possibilite" << endl;
+			}
+			// End of modification
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_OFFERED HDL:" << metaevent.linedev  << endl;
 			trunk->crn = metaevent.crn;
 			event.id = TRUNK_CALL_OFFER;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_ACCEPT:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_ACCEPT HDL:" << metaevent.linedev << endl;
 			event.id = TRUNK_CALL_ACCEPT;
 			trunk->postEvent(&event);
 			break;
 		case GCEV_ANSWERED:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_ANSWERED HDL:" << metaevent.linedev << endl;
 			event.id = TRUNK_OFF_HOOK;
 			trunk->putEvent(&event);
 			break;
 		case GCEV_CONNECTED:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_CONNECTED HDL:" << metaevent.linedev << endl;
 			event.id = TRUNK_CALL_CONNECT;
 			trunk->putEvent(&event);
 			break;
 		case GCEV_DROPCALL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_DROPCALL HDL:" << metaevent.linedev << endl;
 			event.id = TRUNK_CALL_DISCONNECT;
 			trunk->putEvent(&event);
 			break;
 		case GCEV_RELEASECALL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_RELEASECALL HDL:" << metaevent.linedev << endl;
 			trunk->crn = -1;
 			event.id = TRUNK_CALL_RELEASE;
 			event.parm.ok = true;
 			trunk->putEvent(&event);
 			break;
 		case GCEV_RELEASECALL_FAIL:
+            trunk->getName(buffer);
+            slog(Slog::levelDebug) << buffer << ": GCEV_REALESECALL_FAIL HDL:" << metaevent.linedev << endl;
 			trunk->crn = -1;
 			trunk->postGCError("gc_ReleaseCallEx");
 			event.id = TRUNK_CALL_RELEASE;
@@ -749,18 +842,7 @@ void DialogicDriver::run(void)
 			case TDX_PLAYTONE:
 			case TDX_DIAL:
 			case TDX_CALLP:
-				switch(event.id)
-				{
-					case TRUNK_CPA_NOANSWER:
-					case TRUNK_CPA_NODIALTONE:
-					case TRUNK_CPA_NORINGBACK:
-					case TRUNK_CPA_FAILURE:
-					case TRUNK_CPA_BUSYTONE:
-						break;
-
-					default:
-						event.id = TRUNK_TONE_IDLE;
-				}
+				event.id = TRUNK_TONE_IDLE;
 				break;
 			case TDX_PLAY:
 			case TDX_RECORD:
@@ -773,8 +855,6 @@ void DialogicDriver::run(void)
 			event.id = TRUNK_TIMER_EXPIRED;
 			trunk->putEvent(&event);
 			break;
-		case CCEV_SETCHANSTATE:
-			break;
 		default:
 			slog(Slog::levelDebug) << "globalcall: unhandled event 0x" << hex << evtype << endl;
 		}
