@@ -678,6 +678,23 @@ void DialogicTrunk::postGCError(const char *msg)
 	slog(Slog::levelError) << buffer << ": " << msg << " " << gc_error << " " << errormsg << endl;
 }
 
+void DialogicTrunk::postGCError2(const char *msg)
+{
+	char buffer[33];
+	int gc_error;
+	GC_INFO t_Info;
+
+	getName(buffer);
+	gc_error = gc_ErrorInfo(&t_Info);
+	if (gc_error == GC_SUCCESS) {
+		slog(Slog::levelError) << buffer << ": error retrieved OK " << endl;
+		slog(Slog::levelError) << buffer << ": value: " << t_Info.gcValue << " msg: " << t_Info.gcMsg << " add: " << t_Info.additionalInfo << endl;
+	} else {
+		slog(Slog::levelError) << buffer << ": error retrieved KO " << endl;
+	}
+
+}
+
 void DialogicTrunk::exit(void)
 {
 	if(!Trunk::flags.onexit)
@@ -750,6 +767,24 @@ bool DialogicTrunk::waitCall(void)
 		return false;
 }
 
+bool DialogicTrunk::callProgress(int indicator)
+{
+	char buffer[33];
+	int state = -1;
+	getName(buffer);
+	gc_GetCallState(crn, &state);
+	slog(Slog::levelDebug) << buffer << "State:: " << state << endl;
+
+	getName(buffer);
+	slog(Slog::levelDebug) << buffer << "CALLPROGRESS!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
+	slog(Slog::levelDebug) << buffer << "trunk.cpp callProgress: begin" << endl;
+
+	if(gc_CallProgress(crn, indicator) == GC_SUCCESS)
+		return true;
+	else
+		return false;
+}
+
 bool DialogicTrunk::acceptCall(void)
 {
 	if(Trunk::flags.offhook || _accepted)
@@ -800,11 +835,16 @@ bool DialogicTrunk::hangupCall(void)
 		cause = GC_NORMAL_CLEARING;
 	}
 
-	if(gc_DropCall(crn, cause, EV_ASYNC) < 0)
+	if(crn != -1)
 	{
-		postGCError("gc_DropCall");
-		return false;
+		if(gc_DropCall(crn, cause, EV_ASYNC) < 0)
+		{
+			resetChannel();
+			postGCError("gc_DropCall");
+			return false;
+		}
 	}
+	else resetChannel();
 
 	Trunk::flags.offhook = false;
 	return true;
@@ -814,8 +854,16 @@ void DialogicTrunk::releaseCall(void)
 {
 	_disconnecting = false;
 
-	gc_ReleaseCallEx(crn, EV_ASYNC);
-	
+	if(gc_ReleaseCallEx(crn, EV_ASYNC) < 0)
+	{
+		int state;
+
+		gc_GetCallState(crn, &state);
+		slog(Slog::levelDebug) << buffer << "trunk.cpp releaseCall: releasing CRN: " << crn << " FAILED state=" << state << endl;
+		postGCError("gc_ReleaseCallEx");
+	}
+	else slog(Slog::levelDebug) << buffer << "trunk.cpp releaseCall: releasing CRN: " << crn << " OK state=" << state << endl;
+
 	crn = -1;
 }
 
@@ -826,6 +874,10 @@ void DialogicTrunk::resetChannel(void)
 
 bool DialogicTrunk::makeCall(char digit[16], int timeout)
 {
+	char buffer[33];
+	
+	getName(buffer);
+	
 	MAKECALL_BLK makecall_blk;
 	GC_MAKECALL_BLK gclib_blk;
 	int rtn = 0;
@@ -864,8 +916,12 @@ bool DialogicTrunk::makeCall(char digit[16], int timeout)
 
 	rtn = gc_MakeCall(linedev, &crn, digit, &gclib_blk, 0, EV_ASYNC);
 
-	if(rtn < 0)
+	if(rtn < 0) crn = -1;
+	if(crn <= 0)
 	{
+		slog(Slog::levelError) << buffer << ": makeCall error: crn <= 0. Going to reset line" << endl;
+		gc_ResetLineDev(linedev, EV_ASYNC);
+		postGCError2("gc_MakeCall");
 		return false;
 	}
 
