@@ -47,8 +47,22 @@ bool PikaTrunk::flashoffHandler(TrunkEvent *event)
 {
 	switch(event->id)
 	{
-	case TRUNK_CPA_DIALTONE:
+	case TRUNK_CPA_FAILURE:
+		stopDSP();
 		endTimer();
+		if(!TrunkSignal(TRUNK_SIGNAL_CANCEL))
+			TrunkSignal(TRUNK_SIGNAL_STEP);
+		handler = &PikaTrunk::stepHandler;
+		return true;
+	case TRUNK_CPA_DIALTONE:
+		stopDSP();
+		endTimer();
+		if(data.dialxfer.digit)
+		{
+			endTimer();
+			handler = &PikaTrunk::dialHandler;
+			return true;
+		}
 		if(TrunkSignal(TRUNK_SIGNAL_DIALTONE))
 		{
 			endTimer();
@@ -56,14 +70,28 @@ bool PikaTrunk::flashoffHandler(TrunkEvent *event)
 			return true;
 		}
 	case TRUNK_TIMER_EXPIRED:
-		TrunkSignal(TRUNK_SIGNAL_STEP);
+		stopDSP();
+		if(data.dialxfer.digit)
+		{
+			setSymbol(SYM_ERROR, "no-dialtone");
+			TrunkSignal(TRUNK_SIGNAL_ERROR);
+		}
+		else
+			TrunkSignal(TRUNK_SIGNAL_STEP);
 		handler = &PikaTrunk::stepHandler;
 		return true;
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "flashoff");
 		PK_TRUNK_OffHook(hTrk);
 		flags.offhook = true;
-		setTimer(data.flash.offhook);
+		if(!getCPResource())
+		{
+			setSymbol(SYM_ERROR, "dsp-unavailable");
+			TrunkSignal(TRUNK_SIGNAL_ERROR);
+			handler = &PikaTrunk::stepHandler;
+			return true;
+		}
+		setTimer(data.dialxfer.offhook);
 		return true;
 	}
 	return false;
@@ -78,7 +106,7 @@ bool PikaTrunk::flashonHandler(TrunkEvent *event)
 		return true;
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "flashon");
-		getVPResource();
+		stopDSP();
 		setDTMFDetect(false);
 		status[id] = 'f';
 		if(!flags.offhook)
@@ -88,7 +116,7 @@ bool PikaTrunk::flashonHandler(TrunkEvent *event)
 		}
 		PK_TRUNK_OnHook(hTrk);
 		flags.offhook = false;
-		setTimer(data.flash.onhook);
+		setTimer(data.dialxfer.onhook);
 		return true;
 	}
 	return false;
