@@ -46,7 +46,7 @@
 #include <sys/stat.h>
 
 PikaPlay::PikaPlay(PikaTrunk *trunk) :
-Service((Trunk *)trunk, keythreads.priAudio()), AudioFile()
+Service((Trunk *)trunk, keythreads.priAudio()), URLAudio()
 {
 	reset = false;
 	hDsp = trunk->getDevice();
@@ -63,9 +63,9 @@ PikaPlay::~PikaPlay()
 	clrAudio();
 	Terminate();
 
-	sprintf(buffer, "%ld", getPosition());
+	sprintf(buffer, "%ld", getTransfered());
 	trunk->setSymbol(SYM_PLAYED, buffer);
-	Close();
+	URLAudio::Close();
 	if(reset)
 	{
 		PK_AUDIO_Reset(hDsp);
@@ -77,7 +77,19 @@ PikaPlay::~PikaPlay()
 
 char *PikaPlay::getContinuation(void)
 {
-	return getPlayfile();
+	char *fn;
+
+	if(data->play.mode == PLAY_MODE_ONE || 
+           data->play.mode == PLAY_MODE_TEMP)
+		return NULL;
+retry:
+	fn = getPlayfile();
+	if(fn && data->play.mode == PLAY_MODE_ANY)
+	{
+		if(!canAccess(fn))
+			goto retry;
+	}
+	return fn;
 }
 
 timeout_t PikaPlay::Stop(void)
@@ -96,16 +108,20 @@ void PikaPlay::Initial(void)
 {
 	unsigned codec, speed, rate, len, count = 0;
 	char buffer[32];
-	char *fn = getPlayfile();
+	char *fn;
 	struct stat ino;
 	char *ann;
 	struct tm *dt;
 	struct tm tbuf;
 
 	trunk->getName(buffer);
+retry:
+	fn = getPlayfile();
 
 	if(!fn)
 	{
+		if(data->play.mode == PLAY_MODE_ANY)
+			Success();
 		slog(SLOG_ERROR) << buffer << ": no file to play" << endl; 
 		Failure();
 	}
@@ -114,10 +130,23 @@ void PikaPlay::Initial(void)
 	Open(fn);	
 	if(!isOpen())
 	{
+		if(data->play.mode == PLAY_MODE_ANY || 
+		   data->play.mode == PLAY_MODE_ONE)
+			goto retry;
+
 		slog(SLOG_ERROR) << buffer << ": " << fn << ": cannot open" << endl;
 		Failure();
 	}
 
+	if(data->play.mode == PLAY_MODE_TEMP)
+		remove(fn);
+
+	if(data->play.offset)
+		setPosition(data->play.offset);
+
+	if(data->play.limit)
+		setLimit(data->play.limit);
+
 	dt = localtime_r(&ino.st_ctime, &tbuf);
 	sprintf(buffer, "%04d%02d%02d,%02d%02d%02d",
 		dt->tm_year + 1900, dt->tm_mon + 1, dt->tm_mday,
@@ -127,6 +156,8 @@ void PikaPlay::Initial(void)
 	ann = getAnnotation();
 	if(ann)
 		trunk->setSymbol(SYM_ANNOTATION, ann);
+	else
+		trunk->setSymbol(SYM_ANNOTATION, "");
 
 	rate = getSampleRate();
 	switch(getEncoding())
@@ -207,8 +238,9 @@ void PikaPlay::Run(void)
 	
 	// start
 
-	while(!stopped && frame < buffers)
+	while(!stopped && frame < buffers && status == AUDIO_SUCCESS)
 	{	
+		
 		status = getSamples(&buffer[frame * bufsize], samples);
 		control[frame].lpData = (char *)&buffer[frame * bufsize];
 		control[frame].dwBufferLength = bufsize;
@@ -232,6 +264,7 @@ void PikaPlay::Run(void)
 	}
 
 	frame = 0;
+	status = AUDIO_SUCCESS;
 	while(status == AUDIO_SUCCESS && !stopped)
 	{
 		Wait();
@@ -259,24 +292,57 @@ void PikaPlay::Run(void)
 		slog(SLOG_ERROR) << name << ": failed playback" << endl;
 		Failure();
 	}
-	Sleep(delay * 120l);
+	Sleep(delay * 120l + 120);
 	Success();
 }
 
+bool PikaTrunk::exitHandler(TrunkEvent *event)
+{
+	switch(event->id)
+	{
+	case TRUNK_STOP_STATE:
+		return playHandler(event);
+	case TRUNK_ENTER_STATE:
+		debug->DebugState(this, "exit");
+		setTimer(20);
+		return true;
+	case TRUNK_TIMER_EXPIRED:
+		endTimer();
+		event->id = TRUNK_STOP_STATE;
+		return playHandler(event);
+	}
+	return false;
+}		
+
 bool PikaTrunk::playHandler(TrunkEvent *event)
 {
 	switch(event->id)
 	{
-	case TRUNK_AUDIO_IDLE:
+	case TRUNK_STOP_STATE:
 		stopServices();
-	case TRUNK_SERVICE_SUCCESS:
-		TrunkSignal(TRUNK_SIGNAL_STEP);
+		flags.reset = false;
+		endTimer();
 		handler = &PikaTrunk::stepHandler;
 		return true;
+	case TRUNK_AUDIO_IDLE:
+		if(!thread)
+		{
+			setTimer(32);
+			return true;
+		}
+		return true;
+	case TRUNK_SERVICE_SUCCESS:
+//		stopServices();
+		flags.reset = false;
+		TrunkSignal(TRUNK_SIGNAL_STEP);
+		handler = &PikaTrunk::exitHandler;
+		return true;
 	case TRUNK_SERVICE_FAILURE:
+//		stopServices();
+		flags.reset = false;
 		setSymbol(SYM_ERROR, "play-failed");
 		TrunkSignal(TRUNK_SIGNAL_ERROR);
-		handler = &PikaTrunk::stepHandler;
+		handler = &PikaTrunk::exitHandler;
 		return true;
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "play");
@@ -286,8 +352,10 @@ bool PikaTrunk::playHandler(TrunkEvent *event)
 			PK_TRUNK_OffHook(hTrk);
 			flags.offhook = true;
 			setTimer(group->getPickup());
-			return true;
 		}
+		else
+			setTimer(125);
+		return true;
 	case TRUNK_OFF_HOOK:
 		if(thread)
 			return true;
@@ -308,3 +376,46 @@ bool PikaTrunk::playHandler(TrunkEvent *event)
 	return false;
 }
 
+bool PikaTrunk::playwaitHandler(TrunkEvent *event)
+{
+	char buffer[65];
+
+	switch(event->id)
+	{
+	case TRUNK_EXIT_SHELL:
+		if(!tgi.pid)
+			return true;
+		tgi.pid = 0;
+		endTimer();
+		if(event->parm.status)
+		{
+			TrunkSignal(TRUNK_SIGNAL_ERROR);
+			sprintf(buffer, "play-failed-exit-%d", 
+				event->parm.status);
+			setSymbol(SYM_ERROR, buffer);
+			handler = &PikaTrunk::stepHandler;
+			return true;
+		}
+		handler = &PikaTrunk::playHandler;
+		return true;
+	case TRUNK_TIMER_EXPIRED:
+		if(tgi.pid)
+		{
+			kill(tgi.pid, SIGTERM);
+			tgi.pid = 0;
+		}
+		sprintf(buffer, "play-failed-timeout");
+		setSymbol(SYM_ERROR, buffer);
+		TrunkSignal(TRUNK_SIGNAL_ERROR);
+		handler = &PikaTrunk::stepHandler;
+		return true;
+	case TRUNK_ENTER_STATE:
+		debug->DebugState(this, "playwait");
+		endTimer();
+		Trunk::setDTMFDetect();
+		setTimer(data.play.timeout);
+		return true;
+	}
+	return false;
+}
+		
