@@ -74,7 +74,13 @@ bool PikaTrunk::toneHandler(TrunkEvent *event)
 	case TRUNK_ENTER_STATE:
 		debug->DebugState(this, "tone");
 		endTimer();
-		getVPResource();
+		if(!getVPResource())
+		{
+			setSymbol(SYM_ERROR, "dsp-unavailable");
+			TrunkSignal(TRUNK_SIGNAL_ERROR);
+			handler = &PikaTrunk::stepHandler;
+			return true;
+		}
 		Trunk::setDTMFDetect();
 		if(data.tone.wakeup)	
 			setTimer(data.tone.wakeup);
