@@ -384,6 +384,7 @@ bayonne_error *bayonne_request_1_svc(bayonne_request *req, struct svc_req *svc)
 
 bayonne_policy *bayonne_policy_1_svc(char **name, struct svc_req *req)
 {
+	char* value;
 	static char pol_name[ BAYONNE_POLICY_NAME_SZ ];
 	static char pol_sched[ BAYONNE_POLICY_SCHED_SZ ];
 	static char pol_number[ BAYONNE_POLICY_NUMBER_SZ ];
@@ -415,8 +416,12 @@ bayonne_policy *bayonne_policy_1_svc(char **name, struct svc_req *req)
 		return &policy;
 
 	pol->getSchedule( policy.pol_sched );
-	strncpy( policy.pol_name, (char *)pol->getLast("name"), BAYONNE_POLICY_NAME_SZ );
-	strncpy( policy.pol_number, (char *)pol->getLast("number"), BAYONNE_POLICY_NUMBER_SZ );
+	value = (char *)pol->getLast("name");
+	if ( value )
+		strncpy( policy.pol_name, value, BAYONNE_POLICY_NAME_SZ );
+	value = (char *)pol->getLast("number");
+	if ( value )
+		strncpy( policy.pol_number, value, BAYONNE_POLICY_NUMBER_SZ );
 
 	for(id = 0; id < count; ++id)
 	{
@@ -433,6 +438,7 @@ bayonne_policy *bayonne_policy_1_svc(char **name, struct svc_req *req)
 	policy.pol_ports.pol_ports_len = policy.pol_members;			
 	return &policy;
 }
+
 bayonne_port *bayonne_port_1_svc(int *id, struct svc_req *req)
 {
 	static bayonne_port port;
