@@ -191,6 +191,8 @@ static void sayweekday(strstream *s, const char *text)
 		sscanf(text, "%02d/%02d/%04d", &month, &day, &year);
 	else if(text[2] == '.')
 		sscanf(text, "%02d.%02d.%04d", &day, &month, &year);
+	else if(text[4] == '-')
+		sscanf(text, "%04d-%02d-%02d", &year, &month, &day);
 	else
 		sscanf(text, "%04d%02d%02d", &year, &month, &day);
 
@@ -223,6 +225,8 @@ static void saydate(strstream *s, const char *text)
 		sscanf(text, "%02d/%02d/%04d", &month, &day, &year);
 	else if(text[2] == '.')
 		sscanf(text, "%02d.%02d.%04d", &day, &month, &year);
+	else if(text[4] == '-')
+		sscanf(text, "%04d-%02d-%02d", &year, &month, &day);
 	else
 		sscanf(text, "%04d%02d%02d", &year, &month, &day);
 
