@@ -73,14 +73,14 @@ private:
 	void characters(const unsigned char *text, unsigned len);
 	void startElement(const unsigned char *name, const unsigned char **attrib);
 	void endElement(const unsigned char *name);
-	bool loader(trunkdata_t *data);
+	bool loader(Trunk *trk, trunkdata_t *data);
 	const char *mystr(const char *temp);
 
 	BayonneXMLTrunk();
 	~BayonneXMLTrunk();
 };
 
-class BayonneXMLModule : private Module
+class BayonneXMLModule : protected Module
 {
 private:
 	modtype_t getType(void)
@@ -92,22 +92,46 @@ private:
 	TrunkImage *getXML(void)
 		{return (TrunkImage *)new BayonneXMLTrunk;};
 
+	char *dispatch(Trunk *trunk);
+
 public:
 	BayonneXMLModule();
 } bayonnexml;
 
-
 BayonneXMLModule::BayonneXMLModule() : Module()
 {
 	slog(Slog::levelDebug) << "XML: Loading <BayonneXML> interpreter" << endl;
+	driver->addModule(this);
+}
+
+char *BayonneXMLModule::dispatch(Trunk *trunk)
+{
+	trunkdata_t *data = getData(trunk);
+	const char *key, *url = trunk->getKeyword("url");
+
+	data->load.attach = false;
+	data->load.post = false;
+	data->load.section = "#";
+	key = trunk->getKeyword("maxTime");
+	if(!key)
+		key = "60s";
+	data->load.timeout = getSecTimeout(key);
+	data->load.parent = NULL;
+	data->load.gosub = false;
+	data->load.url = trunk->getValue(url);
+	data->load.vars = NULL;
+	if(!data->load.url)
+		return "no-xml-url";
+	data->load.userid[0] = 0;
+	return NULL;
 }
 
 BayonneXMLTrunk::BayonneXMLTrunk() : TrunkImage()
 {
 	if(!buffer)
 	{
-		buffer = new char[512];
 		bufalloc = true;
+		buffer = new char[512];
 	}
 	else
 		bufalloc = false;
@@ -892,13 +916,14 @@ void BayonneXMLTrunk::endLog(void)
 	br = BR_NONE;
 }
 
-bool BayonneXMLTrunk::loader(trunkdata_t *data)
+bool BayonneXMLTrunk::loader(Trunk *trk, trunkdata_t *data)
 {
 	URLStream::Error error;
 	const char *url = data->load.url;
 	const char **vars = (const char **)data->load.vars;
 
-	if(!strnicmp(url, "http:", 5) && vars[0])
+
+	if(!strnicmp(url, "http:", 5) && vars && vars[0])
 	{
 		if(data->load.post)
 			error = post(url, vars);
@@ -918,6 +943,7 @@ bool BayonneXMLTrunk::loader(trunkdata_t *data)
 		slog(Slog::levelError) << "XML: Parsing " << url << " - not well formed" << endl;
 		return false;
 	}
+
 	return true;
 }
 
