@@ -22,8 +22,6 @@ using namespace std;
 #endif
 
 Protocol *Protocol::first = NULL;
-TCPServer *TCPServer::first = NULL;
-unsigned TCPServer::count = 0;
 
 Protocol::Protocol(const char *keyid, tpport_t p) :
 Keydata(keyid), InetHostAddress(), ThreadLock()
@@ -83,32 +81,6 @@ void Protocol::update(InetHostAddress addr)
 	unlock();
 }
 
-TCPServer::TCPServer(InetAddress &bind, tpport_t port, int backlog) :
-TCPSocket(bind, port, backlog)
-{
-	next = first;
-	first = this;
-	++count;
-}
-
-unsigned TCPServer::setPoll(struct pollfd *pfd, TCPServer **svr)
-{
-	unsigned count = 0;
-	TCPServer *node = TCPServer::first;
-
-	while(node)
-	{
-		pfd->fd = node->so;
-		pfd->revents = 0;
-		pfd->events = POLLIN | POLLRDNORM;
-		*svr = node;
-		++svr;
-		++pfd;
-		++count;
-	}
-	return count;
-}
-
 #ifdef	CCXX_NAMESPACES
 };
 #endif
