@@ -3685,6 +3685,16 @@ bool Trunk::scrLibexec(void)
 	if(qc)
 		str << " clid=" << qc;
 
+	qc = getSymbol("session.callednaspos");
+
+	if(qc)
+		str << " session.callednaspos=" << qc;
+
+	qc = getSymbol("pstn.callednaspos");
+	if(qc)
+		str << " pstn.callednaspos=" << qc;
+
+
 	qc = getSymbol(SYM_DIALED);
 	if(qc)
 		str << " dnid=" << qc;
